# ./addons/eventscripts/_lib/python/escinputlib.py

import es


class _InputBoxManager:
   class _EscInputBox:
      def __init__(self, escid, delay, title, text, callback):
         self.escid    = escid
         self.delay    = delay
         self.title    = title
         self.text     = text
         self.callback = callback

      def send(self, userid):
         """ Sends the input box """
         # The last argument is a client command. We supply the client command followed by the id of the instance so we know which input box the user is replying to later.
         # This is only necessary in a library environment, when using only one input box one client command with no default arguments will suffice.
         es.escinputbox(self.delay, userid, self.title, self.text, '_escinputlib %s' % self.escid)

      def _returnData(self, userid, argv):
         """ Returns the data passed with the client command """
         try:
            self.callback(userid, argv)
         finally:
            pass

   def __init__(self):
      self.boxes = {}

   def createInputBox(self, escid, delay, title, text, callback):
      """ Creates and returns an _EscInputBox instance """
      if filter(lambda x: not (x.isalpha() or x.isdigit()), escid): raise ValueError, 'Invalid escid \"%s\" -- escids must be alpha-numeric (no spaces or special characters)' % escid

      if not self.boxes:
         self.registerClientCommand()

      newbox = self.boxes[escid] = self._EscInputBox(escid, delay, title, text, callback)

      return newbox

   def deleteInputBox(self, escid):
      """ Removes an _EscInputBox instance by id """
      if escid in self.boxes:
         del self.boxes[escid]

         if not self.boxes:
            self.unregisterClientCommand()

   def getInputBox(self, escid):
      """ Returns an _EscInputBox instance by id """
      return self.boxes[escid] if escid in self.boxes else None

   def registerClientCommand(self):
      """ Registers the client command "_escinputlib" to the _input_cmd function """
      if not es.exists('clientcommand', '_escinputlib'):
         es.addons.registerBlock('_escinputlib', 'input_cmd', self._input_cmd) # This is only necessary in libraries
         es.regclientcmd('_escinputlib', '_escinputlib/input_cmd', 'Command for retrieving input box data')

   def unregisterClientCommand(self):
      """ Unregisters the "_escinputlib" client command """
      if es.exists('clientcommand', '_escinputlib'):
         es.addons.unregisterBlock('_escinputlib', 'input_cmd') # This is only necessary in libraries
         es.unregclientcmd('_escinputlib')

   def _input_cmd(self):
      """ Gets the instance referred to by the client command and returns the userid and arguments to the callback function """
      box = self.getInputBox(es.getargv(1)) # Since the second argument is the id we use that to find the _EscInputBox instance
      if not box: return

      box._returnData(es.getcmduserid(), map(es.getargv, range(2, es.getargc())))

escinputmanager = _InputBoxManager()


def create(escid, delay, title, text, callback):
   """ Creates and returns an _EscInputBox instance """
   return escinputmanager.createInputBox(escid, delay, title, text, callback)


def delete(escid):
   """ Removes an _EscInputBox instance by id """
   escinputmanager.deleteInputBox(escid)


def get(escid):
   """ Returns an _EscInputBox instance by id """
   return escinputmanager.getInputBox(escid)


def send(escid, userid):
   """ Sends an escinputbox to the userid """
   box = escinputmanager.getInputBox(escid)
   if not box: raise ValueError, 'EscInputBox \"%s\" not found' % escid

   box.send(userid)